% PhaseDiag.m
% ver. 11-NOV-2007 by Alexey Voskov
%
% Calculation of C6H6-C2H5OH phase diagram using two metods - 
% equality of chemical potentials and equality of fugacities. 
% Program has GUI with controls
%
% Component 1 - benzene C6H6
% Component 2 - ethanol C2H5OH
%
% Literature:
% [1] .. , .. , .. 
%     " -", 1987 ; .132
% [2] NIST Chemistry WebBook - http://webbook.nist.gov/chemistry/
% [3] ..   .   -, 1989 .
%
% You may it compile with the command
% mcc -mve PhaseDiag.m
%% ========== PART 1. GRAPHIC USER INTERFACE ==========
function PhaseDiag()
global hOpt_Tx hOpt_Px hOpt_m1 hOpt_m2 hLbl_a00 hChk diag mode
close all;
diag=1; % 1 - Tx diagram, 2 - Px diagram
mode=1; % 1 - chemical potentials, 2 - fugacy
% Create form with controls
hFig=figure('Position', [50,75,600,500]);
col=get(hFig, 'Color');
uicontrol('Style', 'text', 'String', 'Benzene-ethanol phase diagram', ...
          'Position', [75 460 450 35], 'FontSize', 20, 'FontName', 'Arial', ...
          'BackgroundColor',col);
hOpt_Tx=uicontrol('Style', 'radiobutton', 'String', 'T-x diagram', 'Position', [25 225  100 25], ...
                  'Value', 1, 'Callback', 'callback_radiobtn');
hOpt_Px=uicontrol('Style', 'radiobutton', 'String', 'P-x diagram', 'Position', [25 200   100 25], ...
                  'Value', 0, 'Callback', 'callback_radiobtn');
hOpt_m1=uicontrol('Style', 'radiobutton', 'String', 'Chem. potential', 'Position', [25 150 100 25], ...
                  'Value', 1, 'Callback', 'callback_radiobtn');
hOpt_m2=uicontrol('Style', 'radiobutton', 'String', 'Fugacity coeff.', 'Position', [25 125  100 25], ...
                  'Value', 0, 'Callback', 'callback_radiobtn');
hChk = uicontrol('Style', 'checkbox', 'string', 'New window', 'Position', [25 90 100 25]);
hLbl_a00=uicontrol('Style', 'text', 'Position', [25 275 100 25], 'FontSize', 15);

hBut_Go=uicontrol('Style', 'pushbutton', 'String', 'Calculate', 'Position', [25 50 100 25], ...
                  'Callback', 'BuildDiagram');
% Allocate area for phase diagram drawing
hAxes = axes;
set(hAxes, 'Unit', 'pixels', 'Position', [175 50  400 400]);
%
waitfor(hFig);

% ===== Option button callback
function callback_radiobtn
global hOpt_Tx hOpt_Px hOpt_m1 hOpt_m2 hLbl_a00 diag mode
cla; set(hLbl_a00, 'String', '');
switch (gcbo)
    case hOpt_Tx; set (hOpt_Px,'Value',0); diag=1;
    case hOpt_Px; set (hOpt_Tx,'Value',0); diag=2;
    case hOpt_m1; set (hOpt_m2,'Value',0); mode=1;
    case hOpt_m2; set (hOpt_m1,'Value',0); mode=2;
end

%% ========== PART II. PROGRAM LOGIC ==========
function BuildDiagram
global diag hLbl_a00 hChk
% Initialization
x=0:0.05:1;
par0=100;

% 1. Initialization and estimation of parameters
T=55+273.15; p=1;
% T-x diagram
Txy=zeros(length(x),3); Txy(:,2)=x;
Txy_exp=[[80.2, 69.7, 68.3, 68.1, 68.4, 69.0, 71.5, 78.5]+273.15;
         [0,    15.5, 30.2, 50.1, 60.0, 70.0, 84.2, 100]*0.01;
         [0,    33.4, 40.7, 46.4, 50.2, 54.4, 67.5, 100]*0.01]';

% P-x diagram
xyP_exp=[[0,     15.5,  30.2,  50.1,  60.0,  70.0,  84.2,  100]*0.01; %t=55C
         [0,     31.0,  37.4,  42.9,  46.2,  50.6,  63.5,  100]*0.01;
         [326.0, 449.0, 462.8, 463.7, 457.8, 442.6, 397.9, 272.5]/760]';

% Parameter(s) estimation
par = fminsearch(@(arg) ParamFit(Txy_exp, xyP_exp, p, T, arg), par0);

% 2. Building of phase diagram
hold off;
set(hLbl_a00, 'String', sprintf('a=%.0f', par(1)));
if get(hChk, 'Value') ~= 0; figure; end;
[Txy, xyP] = CalcPhDiag(x, T, p, par);
switch diag
  case 1 % T-x diagram
    plot(Txy(:,2), Txy(:,1)-273.15, 'b-', ...
         Txy(:,3), Txy(:,1)-273.15, 'b-', ...
         'LineWidth', 2);
    hold on;
    plot(Txy_exp(:,2), Txy_exp(:,1)-273.15, 'ro', ...
         Txy_exp(:,3), Txy_exp(:,1)-273.15, 'ro', ...
        'LineWidth', 2);
    title('C_6H_6-C_2H_5OH phase diagram');
    xlabel('x(C_2H_5OH)'); ylabel('t,^oC');
    grid on;
  case 2 % P-x diagram
    plot(xyP_exp(:,1), xyP_exp(:,3), 'ro', ...
         xyP_exp(:,2), xyP_exp(:,3), 'ro', ...
        'LineWidth', 2);
    hold on;
    plot(xyP(:,1), xyP(:,3), 'b-', ...
         xyP(:,2), xyP(:,3), 'b-', ...
        'LineWidth', 2);
    title('C_6H_6-C_2H_5OH phase diagram');
    xlabel('x(C_2H_5OH)'); ylabel('p, atm');
    grid on;
end

% ===== Calculate and show phase diagram
% x - 1xN vector 
% T - scalar
% p - scalar
function [Txy, xyP] = CalcPhDiag(x, T, p, par)
% T-x diagram
Txy=zeros(length(x),3); Txy(:,2)=x;
for j=1:length(x)
    Txy(j,1)=fzero( @(Ti) sum(GetP(x(j),Ti,p, par)) - p, 320); 
    pp=GetP(x(j),Txy(j,1),p, par);
    Txy(j,3)=pp(2)./(pp(1)+pp(2));
end
% P-x diagram
xyP=zeros(length(x),3); xyP(:,1)=x;
for j=1:length(x)
    pp=GetP(x(j), T, p, par);
    xyP(j,2)=pp(2)./(pp(1)+pp(2));
    xyP(j,3)=pp(1)+pp(2);
end

% ====== Function is designed for estimation of regular solution parameter
function res = ParamFit(Txy_exp, xyP_exp, p, T, par)
x = Txy_exp(:,2);
[Txy, xyP] = CalcPhDiag(x, T, p, par);
res = sum(Txy(:,3)-Txy_exp(:,3)).^2 + sum(Txy(:,1)-Txy_exp(:,1)).^2 + ... % y - T
      sum(xyP(:,2)-xyP_exp(:,2)).^2 + sum(xyP(:,3)-xyP_exp(:,3)).^2 + ... % y - P
      sum(Txy(:,2)-Txy_exp(:,2)).^2 + sum(Txy(:,1)-Txy_exp(:,1)).^2 + ... % x - T
      sum(xyP(:,1)-xyP_exp(:,1)).^2 + sum(xyP(:,3)-xyP_exp(:,3)).^2;      % x - P

% ====== Returns pressures/fugacities of two components in liquid state
function pcom=GetP(x, T, p, par)
global mode
if x==0; x=0.00001;
elseif x==1; x=0.99999;
end
RT=8.314.*T;
A=( par(1) ) ./ RT;
switch mode
  case 1
    Tr=T./[562.1, 513.9];
    dHvap=[47410*exp(-0.1231*Tr(1))*(1-Tr(1))^0.3602, 
           50430*exp(0.4475*Tr(2))*(1-Tr(2))^0.4989];
    %dG=[30720.*(1-T./353.3); 38560.*(1-T./351.5)]./RT;
    dG=[dHvap(1).*(1-T./353.3); dHvap(2).*(1-T./351.5)]./RT;

    pcom=exp([(log(1-x)+A.*x.^2  -dG(1)), ...
              (log(x)+A.*(1-x).^2-dG(2))]).*p;
  case 2
  p0 = 10.^([4.72583-1660.652./(T-1.461), ...         % p0(C6H6), atm
             5.24677-1598.673./(T-46.424)]./1.01325); % p0(C2H5OH), atm
  c = exp((p-p0)/(0.082058*T).*[0.05839, 0.08937]);   %  
  g = exp([A.*x.^2, A.*(1-x).^2]);
  pcom=p0.*[(1-x),x].*g.*c;
end